﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Analysis.SolutionCaching.RealVectorEncoding {
  [Item("RealVectorMutationPointcut", "Wraps a mutation and applies after-mutation actions.")]
  [StorableClass]
  public class RealVectorMutationPointcut : MutationPointcut<IRealVectorAdvice, IRealVectorManipulator>, IRealVectorManipulator, IRealVectorPointcut {
    public ILookupParameter<RealVector> RealVectorParameter {
      get { return (ILookupParameter<RealVector>)Parameters["RealVector"]; }
    }
    public IValueLookupParameter<DoubleMatrix> BoundsParameter {
      get { return (IValueLookupParameter<DoubleMatrix>)Parameters["Bounds"]; }
    }

    [StorableConstructor]
    protected RealVectorMutationPointcut(bool deserializing) : base(deserializing) { }
    protected RealVectorMutationPointcut(RealVectorMutationPointcut original, Cloner cloner) : base(original, cloner) { }

    public RealVectorMutationPointcut()
      : base() {
      Parameters.Add(new LookupParameter<RealVector>("RealVector", "The real vector that is being manipulating."));
      Parameters.Add(new ValueLookupParameter<DoubleMatrix>("Bounds", "The lower and upper bounds of the real vector."));

      //Would be nicer if it comes from operator collection, but MultiRealVector Crossover does the same thing.
      //Also: MultiRealVectorCrossover cannot be used here as it would generate a cyclic recursion 
      //MultiRealVectorCrossover discovers RealVectorCrossoverPointcut which discovers Multi....
      foreach (Type t in ApplicationManager.Manager.GetTypes(typeof(IRealVectorManipulator))) {
        if ((!typeof(RealVectorMutationPointcut).IsAssignableFrom(t)) &&
          (!typeof(MultiOperator<IRealVectorManipulator>).IsAssignableFrom(t))) {
          MutatorsParameter.ValidValues.Add((IRealVectorManipulator)Activator.CreateInstance(t));
        }
      }

      ParameterizeManipulators();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RealVectorMutationPointcut(this, cloner);
    }

    private void ParameterizeManipulators() {
      foreach (IRealVectorManipulator manipulator in MutatorsParameter.ValidValues.OfType<IRealVectorManipulator>()) {
        manipulator.RealVectorParameter.ActualName = RealVectorParameter.Name;
        manipulator.BoundsParameter.ActualName = BoundsParameter.Name;
      }
      foreach (IStochasticOperator manipulator in MutatorsParameter.ValidValues.OfType<IStochasticOperator>()) {
        manipulator.RandomParameter.ActualName = RandomParameter.Name;
      }
    }
  }
}
